import type {
    ObjectDirective,
} from 'vue'

const nodeList = new Map<Element, Function>();
document.addEventListener('mousedown', (e: MouseEvent) => {
    e.stopPropagation()
    // @ts-ignore
    const paths = e.path as Element[];
    // console.log(paths)
    // 已经绑定过指令的元素
    nodeList.forEach((fn, ele) => {
        // 当前点击元素的数组
        for (const c of paths) {
            // 找到对应的元素 说明是点击了该元素或者他的子元素
            if (c == ele) {
                return;
            }
        }
        // 没有找到 说明在外面 可以出发事件
        fn.call(null)
    })
})

// 定义指令
const clickOutside: ObjectDirective = {
    // 在绑定元素的 attribute 前
    // 或事件监听器应用前调用
    created(el, binding, vnode, prevVnode) {
        // 下面会介绍各个参数的细节
        nodeList.set(el,binding.value)
        // console.log(nodeList)
    },
    // 在元素被插入到 DOM 前调用
    beforeMount(el, binding, vnode, prevVnode) {
    },
    // 在绑定元素的父组件
    // 及他自己的所有子节点都挂载完成后调用
    mounted(el, binding, vnode, prevVnode) {
    },
    // 绑定元素的父组件更新前调用
    beforeUpdate(el, binding, vnode, prevVnode) {
    },
    // 在绑定元素的父组件
    // 及他自己的所有子节点都更新后调用
    updated(el, binding, vnode, prevVnode) {
    },
    // 绑定元素的父组件卸载前调用
    beforeUnmount(el, binding, vnode, prevVnode) {
        nodeList.delete(el)
        // console.log('remove ', index, el)
        // nodeList.splice(index, 1)// 移除要销毁的元素
        // console.log(nodeList)
    },
    // 绑定元素的父组件卸载后调用
    unmounted(el, binding, vnode, prevVnode) {
    }
}

export default clickOutside