import {defineStore} from "pinia";
import {ref} from "vue";
import http from "../util/http";
import message from "../components/message";
import {useRouter} from "vue-router";

// store的id必须唯一
export const useTestStore = defineStore('test-store', {
    state: () => {
        return {
            info: {
                nickname: 'default name'
            }
        }
    },
    getters: {
        nickname: (state) => state.info.nickname
    },
    actions: {
        setNickname(newName) {
            this.info.nickname = newName
        }
    }
})
// 保存数据的key
const TOKEN_KEY = "user-login-token";
export const useUserStore = defineStore('user-store', () => {
    const userinfo = ref<AdminLoginModel>()

    const updateInfo = async () => {
        const data = await http.get<AdminLoginModel>('/admin/user/info')
        userinfo.value = data
    }

    function token() {
        if (userinfo.value && userinfo.value.token) return userinfo.value.token;
        return localStorage.getItem(TOKEN_KEY)
    }

    async function login(params: any) {
        try {
            const data = await http.post<AdminLoginModel>('/admin/user/login', params)
            userinfo.value = data
            localStorage.setItem(TOKEN_KEY, data.token)
        } catch (e) {
            message.toast('登录失败:' + e.message)
            throw e;
        }
    }

    async function logout() {
        const data = await http.get<AdminLoginModel>('/admin/user/info')
        localStorage.removeItem(TOKEN_KEY)
        userinfo.value = null
    }

    return {userinfo, login, logout, updateInfo, token}
})
