package me.xiaoyan.point.api.config;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import me.xiaoyan.point.api.pojo.dto.ApiResult;
import org.springframework.core.MethodParameter;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

import javax.annotation.Resource;
import java.util.List;

@RestControllerAdvice
public class AppResponseConfig implements ResponseBodyAdvice<Object> {
    @Resource
    private ObjectMapper objectMapper;

    @Override
    public boolean supports(MethodParameter methodParameter, Class<? extends HttpMessageConverter<?>> aClass) {
        return true;
    }

    @SneakyThrows
    @Override
    public Object beforeBodyWrite(Object o, MethodParameter methodParameter, MediaType mediaType, Class<? extends HttpMessageConverter<?>> aClass, ServerHttpRequest serverHttpRequest, ServerHttpResponse serverHttpResponse) {
        long traceId = 0;
        List<String> header = serverHttpResponse.getHeaders().get("x-trace-id");
        if (header != null && header.size() > 0){
            String xTraceId = header.get(0);
            if (xTraceId != null) {
                traceId = Long.valueOf(xTraceId);
            }
        }
        if (o instanceof String) {
            return objectMapper.writeValueAsString(ApiResult.success(o).setTraceId(traceId));
        }
        if (o instanceof ApiResult) {
            return ((ApiResult) o).setTraceId(traceId);
        }

        return ApiResult.success(o).setTraceId(traceId);
    }
}