package me.xiaoyan.point.api.config;

import cn.dev33.satoken.exception.NotLoginException;
import me.xiaoyan.point.api.error.BizException;
import me.xiaoyan.point.api.pojo.dto.ApiResult;
import org.springframework.http.HttpStatus;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;

@RestControllerAdvice
public class ResponseExceptionConfig {
    @ExceptionHandler(value = BizException.class) // 要捕获的异常类型
    @ResponseBody
    @ResponseStatus(HttpStatus.OK)
    public ApiResult bizExceptionHandler(HttpServletRequest req, BizException e) {
        return ApiResult.error(e.getCode(), e.getMessage());
    }
    // 参数异常
    @ExceptionHandler(value = MethodArgumentNotValidException.class) // 要捕获的异常类型
    @ResponseBody
    @ResponseStatus(HttpStatus.OK)
    public ApiResult bizExceptionHandler(HttpServletRequest req, MethodArgumentNotValidException e) {
        System.out.println(e.getFieldError().getDefaultMessage());
        return ApiResult.error(1201, e.getBindingResult().getAllErrors().get(0).getDefaultMessage());
    }

    // 没有单独处理的异常都会被此方法处理
    @ExceptionHandler(value = Exception.class) // 要捕获的异常类型
    @ResponseBody
    @ResponseStatus(HttpStatus.OK)
    public ApiResult exceptionHandler(HttpServletRequest req, Exception e) {
        if(e instanceof NotLoginException){
            return ApiResult.error(403,"登录凭证无效");
        }
        e.printStackTrace();
        return ApiResult.error(-1, "App Internal Error:" + e.getMessage());
    }
}
