package me.xiaoyan.point.api.controller;

import cn.dev33.satoken.stp.StpUtil;
import io.swagger.annotations.ApiOperation;
import lombok.SneakyThrows;
import me.xiaoyan.point.api.error.BizException;
import me.xiaoyan.point.api.pojo.UserInfo;
import me.xiaoyan.point.api.pojo.dto.UserInfoWithToken;
import me.xiaoyan.point.api.pojo.vo.SignResult;
import me.xiaoyan.point.api.pojo.vo.UserLoginData;
import me.xiaoyan.point.api.service.UserInfoService;
import org.springframework.beans.BeanUtils;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

@RestController
@RequestMapping("user")
public class UserinfoController {
    @Resource
    private UserInfoService userInfoService;

    // code => openid等信息
    // 使用wx.login获取到的code进行登录，完成登录后下发用户登录凭证
    @ApiOperation("登录")
    @PostMapping("login")
    @SneakyThrows
    public String login(@Validated @RequestBody UserLoginData data) {
        //@NotNull(message = "登录code不能为空")
        UserInfo info = userInfoService.login(data);
        // 完成登录
        StpUtil.login(info.getId());
        // 可以考虑将用户信息保存到sa-token
//        StpUtil.getSession().set("userinfo",info);
        return StpUtil.getTokenInfo().getTokenValue();
    }

    @ApiOperation("获取登录用户的基本信息")
    @RequestMapping("info")
    public UserInfoWithToken getInfo(String code) {
        UserInfoWithToken data = new UserInfoWithToken();
        UserInfo info;
        if (StringUtils.hasLength(code)) {
            info = userInfoService.getInfoByCode(code);
            if (info == null) {
                throw BizException.create("用户不存在");
            }
            StpUtil.login(info.getId());
            String token = StpUtil.getTokenInfo().getTokenValue();
            data.setToken(token);
        }else{
            int uid = StpUtil.getLoginIdAsInt();
            info = userInfoService.getInfoById(uid);
        }
        BeanUtils.copyProperties(info,data);
        return data;
    }
}
