package me.xiaoyan.point.api.controller.admin;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import me.xiaoyan.point.api.error.BizException;
import me.xiaoyan.point.api.pojo.Goods;
import me.xiaoyan.point.api.pojo.vo.GoodsQueryParam;
import me.xiaoyan.point.api.pojo.vo.PageDataResult;
import me.xiaoyan.point.api.pojo.vo.PageParam;
import me.xiaoyan.point.api.service.GoodsService;
import me.xiaoyan.point.api.util.DataStatus;
import org.apache.ibatis.annotations.Delete;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

@RestController
@RequestMapping("admin/goods")
public class GoodsAdminController {
    @Resource
    private GoodsService goodsService;

    @PostMapping("/list")
    public PageDataResult list(@RequestBody GoodsQueryParam param) {
        return PageDataResult.convert(goodsService.queryByPage(param));
    }

    // 新增
    @PostMapping
    public Goods create(@RequestBody Goods goods) {
        if (!goodsService.save(goods)) {
            throw BizException.saveFail();
        }
        return goods;
    }

    // 修改
    @PutMapping("/{id}")
    public Goods update(@RequestBody Goods goods, @PathVariable("id") long id) {
        goods.setId(id);
        if (!goodsService.updateById(goods)) {
            throw BizException.saveFail();
        }
        return goods;
    }

    // 查询
    @GetMapping("/{id}")
    public Goods get(@PathVariable("id") long id) {
        return goodsService.getById(id);
    }

    // 删除
    @DeleteMapping("/{id}")
    public boolean remove(@PathVariable("id") long id) {
        if (id < 1) throw BizException.paramError();
        // 只是逻辑删除 -> 将状态更新未删除即可

        return goodsService.updateById(
                Goods.builder()
                        .id(id)
                        .status(DataStatus.DELETE) // 状态设置为删除
                        .build()
        );
    }
}
