package me.xiaoyan.point.api.controller.admin;

import me.xiaoyan.point.api.error.BizException;
import me.xiaoyan.point.api.pojo.OrderInfo;
import me.xiaoyan.point.api.pojo.vo.OrderQueryParam;
import me.xiaoyan.point.api.pojo.vo.PageDataResult;
import me.xiaoyan.point.api.service.OrderInfoService;
import me.xiaoyan.point.api.util.DataStatus;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

@RestController
@RequestMapping("/admin/order")
public class OrderAdminController {
    @Resource
    private OrderInfoService orderInfoService;


    @PostMapping("/list")
    public PageDataResult list(@RequestBody OrderQueryParam param) {
        return PageDataResult.convert(orderInfoService.queryByPage(param,false,true));
    }

    // 修改
    @PutMapping("/{id}")
    public OrderInfo update(@RequestBody OrderInfo info, @PathVariable("id") String id) {
        info.setId(id);
        if (!orderInfoService.updateById(info)) {
            throw BizException.saveFail();
        }
        return info;
    }

    // 查询
    @GetMapping("/{id}")
    public OrderInfo get(@PathVariable("id") String id) {
        return orderInfoService.getById(id);
    }

    // 删除
    @DeleteMapping("/{id}")
    public boolean remove(@PathVariable("id") String id) {
        if (!StringUtils.hasText(id)) throw BizException.paramError();
        // 只是逻辑删除 -> 将状态更新未删除即可

        return orderInfoService.updateById(
                OrderInfo.builder()
                        .id(id)
                        .status(DataStatus.DELETE) // 状态设置为删除
                        .build()
        );
    }

    @PutMapping("/{id}/status")
    public boolean setStatus(@PathVariable("id") String id, int status) {
        if (!StringUtils.hasText(id)) throw BizException.paramError();
        return orderInfoService.updateById(
                OrderInfo.builder()
                        .id(id)
                        .status(status)
                        .build()
        );
    }
}
