package me.xiaoyan.point.api.controller.admin;

import cn.dev33.satoken.stp.StpUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import me.xiaoyan.point.api.error.BizException;
import me.xiaoyan.point.api.pojo.UserInfo;
import me.xiaoyan.point.api.pojo.vo.GoodsQueryParam;
import me.xiaoyan.point.api.pojo.vo.PageDataResult;
import me.xiaoyan.point.api.pojo.vo.UserAdminInfo;
import me.xiaoyan.point.api.pojo.vo.UserQueryParam;
import me.xiaoyan.point.api.service.UserInfoService;
import me.xiaoyan.point.api.util.DataStatus;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.HashMap;

/**
 * 管理员操作控制器示例
 * Create by callmeyan
 * @module demo/point_sys_api/admin
 * @folder demo/point_sys_api/api
 */
@RestController
@RequestMapping("admin/user")
@Slf4j
public class UserAdminController {
    private HashMap<String, UserAdminInfo> userStoreMap = new HashMap<>();

    @Resource
    private UserInfoService userInfoService;

    @PostConstruct
    public void initUserStore() {
        log.info("初始化用户数据");
        userStoreMap.put("admin", UserAdminInfo.create(1, "admin", "admin"));
        userStoreMap.put("test", UserAdminInfo.create(2, "test", "123123"));
    }

    @PostMapping
    public UserInfo update(@RequestBody UserInfo userInfo) {
        if (!userInfoService.updateById(userInfo)) throw BizException.saveFail();
        return userInfo;
    }

    /**
     * 用户登录
     * @status released
     * @folder demo/point_sys_api/api/admin
     * @param user
     * @return
     */
    @SneakyThrows
    @PostMapping("login")
    public UserAdminInfo login(@Validated @RequestBody UserAdminInfo user) {
        Thread.sleep(2000);
        // 判断是否存在账号
        if (!userStoreMap.containsKey(user.getAccount())) {
            throw BizException.create("账号不存在");
        }
        // 获取到用户数据
        UserAdminInfo admin = userStoreMap.get(user.getAccount());
        // 判断密码
        if (!admin.getPassword().equals(user.getPassword())) {
            throw BizException.create("密码不正确");
        }
        user.setId(admin.getId());
        user.setPassword("");
        StpUtil.login(user.getId(), "admin");
        user.setToken(StpUtil.getTokenInfo().getTokenValue());
        StpUtil.getSession().set("user", user);
        return user;
    }

    @RequestMapping("info")
    public UserAdminInfo info() {
        return (UserAdminInfo) StpUtil.getSession().get("user");
    }

    @RequestMapping("logout")
    public String logout() {
        StpUtil.logout();
        return "ok";
    }

    @PostMapping("/list")
    public PageDataResult list(@RequestBody UserQueryParam param) {
        return PageDataResult.convert(userInfoService.queryByPage(param));
    }

    @GetMapping("/remove")
    public boolean remove(int id) {
        return userInfoService.updateById(UserInfo.builder().id(id).status(DataStatus.DELETE).build());
    }
}
