package me.xiaoyan.point.api.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import me.xiaoyan.point.api.error.BizException;
import me.xiaoyan.point.api.mapper.PointRecordMapper;
import me.xiaoyan.point.api.pojo.Point;
import me.xiaoyan.point.api.pojo.PointRecord;
import me.xiaoyan.point.api.pojo.vo.PageParam;
import me.xiaoyan.point.api.service.PointRecordService;
import me.xiaoyan.point.api.service.PointService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

@Service
public class PointRecordServiceImpl extends ServiceImpl<PointRecordMapper, PointRecord>
        implements PointRecordService {
    @Resource
    private PointService pointService;
    @Transactional
    @Override
    public PointRecord record(Integer uid, Integer point, String reason) {
        if(point == 0 ){
            throw BizException.create("积分值没有变化");
        }
        // 查询用户的积分信息
        final Point pointInfo = pointService.getById(uid);
        if(point < 0 && pointInfo.getTotalPoint() < Math.abs(point)){
            throw BizException.create("积分不足");
        }

        final PointRecord record = PointRecord.builder()
                .uid(uid)
                .point(point)
                .reason(reason)
                .currentTotalPoint(pointInfo.getTotalPoint())
                .build();
        pointInfo.setTotalPoint(
                pointInfo.getTotalPoint() + point
        );
        // 更新
        pointService.updateById(pointInfo);
        this.save(record);
        return record;
    }

    @Override
    public IPage<PointRecord> histories(Integer uid, PageParam param) {
        QueryWrapper<PointRecord> q = new QueryWrapper();
        q.eq("uid",uid);
        this.getBaseMapper().selectPage(param.getPage(), q);
        return null;
    }

    public IPage<PointRecord> exchangeHistories(Integer uid, PageParam param) {
        QueryWrapper<PointRecord> q = new QueryWrapper();
        q.eq("uid",uid);
        q.lt("point",0);
        this.getBaseMapper().selectPage(param.getPage(), q);
        return null;
    }
}
