package me.xiaoyan.point.api.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import me.xiaoyan.point.api.error.BizException;
import me.xiaoyan.point.api.mapper.SignRecordMapper;
import me.xiaoyan.point.api.pojo.SignRecord;
import me.xiaoyan.point.api.pojo.vo.SignResult;
import me.xiaoyan.point.api.service.PointRecordService;
import me.xiaoyan.point.api.service.SignRecordService;
import me.xiaoyan.point.api.util.DateUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

@Service
public class SignRecordServiceImpl extends ServiceImpl<SignRecordMapper, SignRecord>
        implements SignRecordService {

    @Resource
    private PointRecordService pointRecordService;

    public boolean todayIsSign(int uid) {
        return this.getBaseMapper().signCountByDate(uid, DateUtils.today()) > 0;
    }

    @Transactional
    @Override
    public SignResult signToday(int uid) {
        if (todayIsSign(uid)) throw BizException.create("今日已经签过了");
        // 计算连续的签到天数 连续签到积分的公式 = 天数 * 5
        int days = continuousDays(uid, true);
        // TODO 完成签到
        SignRecord record = SignRecord.builder()
                .uid(uid)
                .point(5 * days)
                .build();
        pointRecordService.record(uid, 5 * days, "签到" + days + "天，获得积分");
        save(record);
        // 封装签到信息
        return SignResult.builder()
                .signToday(true)
                .continuousDays(days)
                .build();
    }

    public int continuousDays(int uid, boolean isSign) {
        // 如果是签到数据则只查6天，否则查询7天
        final List<SignRecord> signRecords = this.getBaseMapper().selectRecentDaysRecord(uid, isSign ? 6 : 7);
        List<Date> dates = new ArrayList<>();
        if (isSign) {
            dates.add(new Date());
        }else{
            // 没有记录直接返回0
            if(signRecords.size() == 0) return 0;
            Date firstDate = signRecords.get(0).getCreateTime();
            // 第一个记录是否是今天
            if(!DateUtils.isToday(firstDate) && !DateUtils.isYesterday(firstDate)){
                return 0;
            }
        }
        signRecords.forEach(it -> dates.add(it.getCreateTime()));
        Date[] dateArr = new Date[dates.size()];
        return DateUtils.continuousDays(dates.toArray(dateArr));
    }

    @Override
    public SignResult info(int uid) {
        return SignResult.builder()
                .signToday(todayIsSign(uid)) // 今天是否签到
                .continuousDays(continuousDays(uid, false))
                .build();
    }


}
