package me.xiaoyan.point.api.util;

import lombok.SneakyThrows;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

public class DateUtils {
    private final static SimpleDateFormat full = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    private final static SimpleDateFormat date = new SimpleDateFormat("yyyy-MM-dd");

    public static String today() {
        return date.format(new Date());
    }

    public static String now() {
        return full.format(new Date());
    }

    public static String formatDate(Date d) {
        return date.format(d);
    }

    public static String formatDatetime(Date d) {
        return full.format(d);
    }

    @SneakyThrows
    public static Date parse(String str) {
        return full.parse(str);
    }

    public static int continuousDays(Date[] dates) {
        if(dates.length == 0){
            return 0;
        }
        int totalDays = 1;
        for (int i = 1;i < dates.length; i ++){
            Date d1 = dates[i-1];
            Date d2 = dates[i];
            if(!isBetween(d1,d2)){
                break;
            }
            totalDays ++;
        }
        return totalDays;
    }

    public static boolean isToday(String d){
        try {
            return isToday(full.parse(d));
        } catch (ParseException e) {
            return false;
        }
    }
    public static boolean isToday(Date d){
        if(d == null) return false;
        return today().equalsIgnoreCase(formatDate(d));
    }

    public static boolean isYesterday(Date d){
        if(d == null) return false;
        Date today = new Date(); // 今天
        if(today.getTime() < d.getTime()) return false;
        return isBetween(today,d);
    }

    public static boolean isBetween(Date d1,Date d2){
        // 直接获取两个日期之间的毫秒,计算
        Date date1 = parse(formatDate(d1) + " 00:00:00");
        Date date2 = parse(formatDate(d2) + " 00:00:00");
        // 两个日期之间相差的毫秒数
        long diffTimes = Math.abs(date1.getTime() - date2.getTime());
        return diffTimes <= 24 * 3600 * 1000;
    }
}
