package me.xiaoyan.point.api.util;

import com.google.gson.Gson;
import me.xiaoyan.point.api.pojo.dto.WechatUserInfo;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.springframework.util.StringUtils;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.security.Key;
import java.security.Security;
import java.util.Arrays;
import java.util.Base64;

/**
 * 微信工具类
 */
public class WechatDecryptDataUtil {

    /**
     *
     * @param encryptDataB64 加密的数据 从wx.getUserProfile而来
     * @param sessionKeyB64 解密密钥,从code2Session而来
     * @param ivB64
     * @return
     */
    public static WechatUserInfo decryptData(String encryptDataB64, String sessionKeyB64, String ivB64) {
        System.out.println("encryptDataB64:" + encryptDataB64);
        System.out.println("sessionKeyB64:" +sessionKeyB64);
        System.out.println("ivB64:" + ivB64);
        String result =  new String(
                decryptOfDiyIV(
                        Base64.getDecoder().decode(encryptDataB64),
                        Base64.getDecoder().decode(sessionKeyB64),
                        Base64.getDecoder().decode(ivB64)
                )
        );
        System.out.println(result);
        if(!StringUtils.hasLength(result)){
            throw new RuntimeException("解码失败");
        }
        return (new Gson()).fromJson(result,WechatUserInfo.class);
    }

    private static final String KEY_ALGORITHM = "AES";
    private static final String ALGORITHM_STR = "AES/CBC/PKCS7Padding";
    private static Key key;
    private static Cipher cipher;

    private static void init(byte[] keyBytes) {
        // 如果密钥不足16位，那么就补足.  这个if 中的内容很重要
        int base = 16;
        if (keyBytes.length % base != 0) {
            int groups = keyBytes.length / base + (keyBytes.length % base != 0 ? 1 : 0);
            byte[] temp = new byte[groups * base];
            Arrays.fill(temp, (byte) 0);
            System.arraycopy(keyBytes, 0, temp, 0, keyBytes.length);
            keyBytes = temp;
        }
        // 初始化
        Security.addProvider(new BouncyCastleProvider());
        // 转化成JAVA的密钥格式
        key = new SecretKeySpec(keyBytes, KEY_ALGORITHM);
        try {
            // 初始化cipher
            cipher = Cipher.getInstance(ALGORITHM_STR, "BC");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 解密方法
     *
     * @param encryptedData 要解密的字符串
     * @param keyBytes      解密密钥
     * @param ivs           自定义对称解密算法初始向量 iv
     * @return 解密后的字节数组
     */
    private static byte[] decryptOfDiyIV(byte[] encryptedData, byte[] keyBytes, byte[] ivs) {
        byte[] encryptedText = null;
        init(keyBytes);
        try {
            cipher.init(Cipher.DECRYPT_MODE, key, new IvParameterSpec(ivs));
            encryptedText = cipher.doFinal(encryptedData);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return encryptedText;
    }

}
